/*
 * cgi.c - cgi interface functions
 *
 * $Id$
 *
 * Copyright (c) 2001, The Colocation Corporation. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.  Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.  Neither the name of the The
 * Colocation Corporation nor the names of its contributors may be used to
 * endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * This software was originally written under grants from the Colocation
 * Corporation. To find the most recent versions of the software, please visit
 * http://www.loligo.com/ocoloc  To contribute patches or for other questions 
 * please contact jtodd@loligo.com
 *
 * To find out more about the Colocation Corporation, see http://www.coloco.com/ 
 *                                
 */
        

#include <cgi-lib.h>
#include "ocoloc.h"

void
get_cgi_params(Options * opt)
{
   LIST           *cgivars;
   char           *device, *ifc;
   char           *units;
   char           *duration;
   char           *ch;

   if (!(cgivars = cgi_input_parse()))
      return;

   /* list_print(cgivars); */

   if (!(strcmp((device = find_val(cgivars, "ifc")), ""))) {
      printf("<tt>invalid Port selected</tt><p>\n");
      return;
   }

   if ( (ch = strchr(device, '(')))
      *(ch - 1) = '\0';

   ch = strchr(device, ':');
   *ch = 0;
   ifc = ch + 1;

   opt->device = strdup(device);
   opt->interface = strdup(ifc);

   units = find_val(cgivars, "units");
   if (!strcmp(units, "bits")) {
      opt->units = bits;
      opt->unit_name = strdup("Bits");
   } else if (!strcmp(units, "bytes")) {
      opt->units = bytes;
      opt->unit_name = strdup("Bytes");
   } else if (!strcmp(units, "kilobytes")) {
      opt->units = kilobytes;
      opt->unit_name = strdup("Kilobytes");
   }

   duration = find_val(cgivars, "duration");
   if (strcmp(duration, "(none)")) {
      opt->end = NOW;
      if (!strcmp(duration, "epoch"))
	 opt->start = 0;
      else if (!strcmp(duration, "hour"))
	 opt->start = opt->end - 60 * 60;
      else if (!strcmp(duration, "day"))
	 opt->start = opt->end - 60 * 60 * 24;
      else if (!strcmp(duration, "week"))
	 opt->start = opt->end - 60 * 60 * 24 * 7;
      else if (!strcmp(duration, "month"))
	 opt->start = opt->end - 60 * 60 * 24 * 7 * 30;

   } else {

      time_t          now = NOW;
      struct tm      *tms = localtime(&now);
      struct tm       mytm;
      int             month, year, day, ldom;
      if ((year = atoi(find_val(cgivars, "fy"))) < 1) {
	 printf("<tt>missing from year </tt><p>\n");
	 return;
      }

      if (((month = atoi(find_val(cgivars, "fm"))) < 1) || month > 12) {
	 printf("<tt>missing/bad from month </tt><p>\n");
	 return;
      }

      switch (month) {
      case 9:
      case 4:
      case 6:
      case 11:
	 ldom = 30;
	 break;
      case 2:
	 ldom = (year % 4 == 0 ? 29 : 28);
	 break;
      default:
	 ldom = 31;
      }

      if (((day = atoi(find_val(cgivars, "fd"))) < 1) || day > ldom) {
	 printf("<tt>missing/bad from day </tt><p>\n");
	 return;
      }
      memset(&mytm, 0, sizeof(struct tm));
      mytm.tm_isdst = tms->tm_isdst;
      mytm.tm_zone = strdup(tms->tm_zone);
      mytm.tm_gmtoff = tms->tm_gmtoff;

      mytm.tm_mon = atoi(find_val(cgivars, "fm")) - 1;
      mytm.tm_mday = atoi(find_val(cgivars, "fd"));
      mytm.tm_year = atoi(find_val(cgivars, "fy")) + 100;
      opt->start = mktime(&mytm);

      if (!atoi(find_val(cgivars, "tm")) &&
	  !atoi(find_val(cgivars, "td")) &&
	  !atoi(find_val(cgivars, "ty")))
	 opt->end = NOW;
      else {

	 if (!find_val(cgivars, "td") || !atoi(find_val(cgivars, "td")) ||
	     !find_val(cgivars, "tm") || !atoi(find_val(cgivars, "tm")) ||
	     !find_val(cgivars, "ty") || !atoi(find_val(cgivars, "ty"))) {
	    printf("<tt>missing/bad date component</tt><p>\n");
	    return;
	 }
	 mytm.tm_mon = atoi(find_val(cgivars, "tm")) - 1;
	 mytm.tm_mday = atoi(find_val(cgivars, "td"));
	 mytm.tm_year = atoi(find_val(cgivars, "ty")) + 100;
	 opt->end = mktime(&mytm);

      }

   }

   if (opt->start > NOW) {
      printf("Start Date in the Future (invalid).");
      return;
   }

   if (opt->end > NOW) {
      printf("End Date in the Future (invalid).");
      return;
   }

   opt->use_cgi = 1;

}

void
send_html_header(DevList * tree)
{
   DevList        *h = tree;
   IfList         *i;
   printf("Content-Type: text/html\n\n");
   printf("<html><head><title>Ocoloc</title></head>\n<body>\n");

   printf("<h3><tt>ColoCo Bandwidth Usage Report</tt></h3>\n");
   printf("<form name=\"q\">\n");
   printf("<table border=1 width=480>\n");

   printf("<tr><td colspan=2>Port</td><td>Units</td></tr>\n");

   printf("<tr><td colspan=2><select name=\"ifc\">\n");
   while (h) {
      i = h->interface;
      while (i) {
	 printf("\t\t<option>%s:%s", h->name, i->name);
	 if (i->comment)
	    printf(" (%s)", i->comment);
         printf("</option>\n");
	 i = i->next;
      }
      h = h->next;
   };

   printf("\t</select></td>\n");

   printf("\t<td><select name=\"units\">\n");
   printf("\t\t<option>bits</option>\n");
   printf("\t\t<option>bytes</option>\n");
   printf("\t\t<option selected>kilobytes</option>\n");
   printf("\t</select></td></tr>\n\n");

   printf("<tr><td>From (month/day/year)</td>");
   printf("<td>To (blank=now)</td><td>or the last</td></tr>");
   printf("<tr><td>\n");
   printf("\t<input maxLength=2 size=2 name=fm value=" "> /\n");
   printf("\t<input maxLength=2 size=2 name=fd value=" "> / 20\n");
   printf("\t<input maxLength=2 size=2 name=fy value=" ">\n");
   printf("</td><td>\n");
   printf("\t<input maxLength=2 size=2 name=tm value=" "> / \n");
   printf("\t<input maxLength=2 size=2 name=td value=" "> / 20\n");
   printf("\t<input maxLength=2 size=2 name=ty value=" ">\n");
   printf("</td>\n");
   printf("\t<td><select name=\"duration\">\n");
   printf("\t\t<option>hour</option>\n");
   printf("\t\t<option>day</option>\n");
   printf("\t\t<option>week</option>\n");
   printf("\t\t<option selected>month</option>\n");
   printf("\t\t<option>epoch</option>\n");
   printf("\t</select></td></tr>\n\n");

   printf("<tr><td colspan=3 align=center>");
   printf("<input type=\"submit\" name=\"Submit\" value=\"Submit\">\n");
   printf("</td></tr></form><p>\n");

   printf("</td></tr></table>\n");
   printf("<br><hr>\n");
}

void
send_html_footer(Options * opt)
{
   printf("<p><hr><br>\n");
   printf("(c)2002 Coloco Inc.<p>\n");
   printf("</body></html>\n\n");
}

void
write_html_stats(Stats * s)
{

   printf("<table border=1 width=480>\n");
   printf("<tr><td>Host</td><td>%s</td></tr>\n", s->hostname);
   printf("<tr><td>Interface</td><td>%s</td></tr>\n", s->ifname);
   printf("<tr><td>Start Time</td><td>%s</td></tr>\n", ctime(&s->start));
   printf("<tr><td>End Time</td><td>%s</td></tr>\n", ctime(&s->end));
   if (s->ifcomment)
      printf("<tr><td>Comment</td><td>%s</td></tr><br>\n", s->ifcomment);
   printf("<tr><td>Units</td><td>%s</td></tr>\n", s->units);
   printf("</table><p>\n");

   printf("<table border=1 width=480>\n");
   printf("<tr><td></td><td>min</td><td>max</td>\n");
   printf("<tr><td>Aggregate Throughput</td><td>%ld</td><td>%ld</td>\n", 
	s->min_thru, s->max_thru);
   printf("</table><p>\n");

   printf("<table border=1 width=480>\n");
   printf("<tr><td></td><td>in</td><td>out</td><td>agg</td>\n");
   printf("<tr><td>%2.0f%% Sample</td><td>%lu</td><td>%lu</td><td>%lu</td></tr>\n", 
	(s->pct * 100), s->wgt_in, s->wgt_out, s->wgt_agg);
   printf("</table><p>\n");

   printf("<table border=1 width=480>\n");

   printf("<tr><td/><td>in</td><td>out</td>\n");
   printf("<tr><td>Raw Average</td><td>");
   mpz_out_str(stdout, 10, s->avg_in);
   printf("</td><td>");
   mpz_out_str(stdout, 10, s->avg_out);
   printf("</td></tr>\n");

   printf("<tr><td>Raw Totals</td><td>");
   mpz_out_str(stdout, 10, s->ttl_in);
   printf("</td><td>");
   mpz_out_str(stdout, 10, s->ttl_out);
   printf("</td></tr>\n");

   printf("</table><p>\n");

}
