/*
 * main.c - main program for ocoloc
 * 
 * $Id$
 * 
 * Copyright (c) 2001, The Colocation Corporation. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.  Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.  Neither the name of the The
 * Colocation Corporation nor the names of its contributors may be used to
 * endorse or promote products derived from this software without specific
 * prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 * This software was originally written under grants from the Colocation
 * Corporation. To find the most recent versions of the software, please visit
 * http://www.loligo.com/ocoloc  To contribute patches or for other questions 
 * please contact jtodd@loligo.com
 *
 * To find out more about the Colocation Corporation, see http://www.coloco.com/ 
 *                                
 */


#include "ocoloc.h"

unsigned long gvBailSysLogPri;
int Debug = 0;

char           *report_usage =
"\n"
"ocoloc $Id$ - A bandwidth utilization measurement tool\n"
"Developed by Coloco, Inc. - see http://www.coloco.com/public/ for updates\n"
"\n"
"  usage: report[.cgi] options:\n"
"options: -s start date/time (fmt: MMDDYY-hhmm) default = epoch\n"
"         -e end date/time (fmt: MMDDYY-hhmm) default = now\n"
"         -d device hostname\n"
"         -i interface (port name)\n"
"         -r recurse all devices and interfaces\n"
"         -u units (b = bits, B = Bytes, k = kilobits)\n"
"	  -k don't fork on poll\n"
"\n";

time_t
convert_time(char *ts)
{
   time_t          now = NOW;
   time_t          then;
   struct tm      *nowtm = localtime(&now);
   struct tm       mytm;

   /* make sure we have valid input */
   if (strlen(ts) != 11 || ts[6] != '-')
      BAIL("Incorrect Date Format for %s\n", ts);

   /* get the current time (so I can use the TZ, etc */
   memset(&mytm, 0, sizeof(struct tm));
   mytm.tm_isdst = nowtm->tm_isdst;
   mytm.tm_zone = strdup(nowtm->tm_zone);
   mytm.tm_gmtoff = nowtm->tm_gmtoff;

   /* fill in the blanks, remember ASCII people! */
   mytm.tm_mon = ((ts[0] - '0') * 10) + (ts[1] - '0') - 1;
   mytm.tm_mday = ((ts[2] - '0') * 10) + (ts[3] - '0');
   mytm.tm_year = (2000 + ((ts[4] - '0') * 10) + (ts[5] - '0')) - 1900;
   mytm.tm_hour = ((ts[7] - '0') * 10) + (ts[8] - '0');
   mytm.tm_min = ((ts[9] - '0') * 10) + (ts[10] - '0');
   then = mktime(&mytm);

   /* bail on mistake */
   if (then == -1)
      BAIL("Incorrect Date Specified for %s\n", ts);

   if (then > now + 60)
      BAIL("Invalid (future) Date Specified for %s\n", ts);

   return then;
}

int
main(int argc, char **argv)
{

   Defaults        defaults;
   DevList        *host_tree = 0;
   Options         options;
   char           *pn = *argv;
   char            ch;
   int             recurse = 0;

   while (strchr(pn, '/')) {
      pn = strchr(pn, '/'); 
      ++pn;
      }

   memset(&defaults, 0, sizeof(defaults));
   memset(&options, 0, sizeof(options));
   options.config_file = strdup("./ocoloc.conf");
   options.units = kilobytes;
   options.unit_name = strdup("kB");

   while ((ch = getopt(argc, argv, "c:d:D:e:f:i:ks:tu:r?")) != -1)
      switch (ch) {
      case 'c':
	 options.config_file = strdup(optarg);
	 break;
      case 'd':
	 options.device = strdup(optarg);
	 break;
      case 'D':
	 Debug = atoi(optarg);
	 break;
      case 'e':
	 options.end = convert_time(optarg);
         printf("  end time selected is %s", ctime(&options.end));
	 break;
      case 'f':
	 options.config_file = strdup(optarg);
	 break;
      case 'k':
	 options.dont_fork = 1;
	 break;
      case 'i':
	 if (!options.device)
	    BAIL("%s: set a device (-d) before the interface (-i)", "error");
	 options.interface = strdup(optarg);
	 break;
      case 's':
	 options.start = convert_time(optarg);
         printf("start time selected is %s", ctime(&options.start));
         break;
      case 'r':
	 recurse = 1;
	 break;
      case 't':
	 options.show_parse++;
	 break;
      case 'u':
	 if (*optarg == 'b') {
	    options.units = bits;
	    options.unit_name = strdup("b");
	 } else if (*optarg == 'B') {
	    options.units = bytes;
	    options.unit_name = strdup("B");
	 } else if (*optarg == 'k') {
	    options.units = kilobytes;
	    options.unit_name = strdup("kB");
	 } else
	    BAIL("unknown units specified. use b=bits, B=bytes, k=kilobytes"); 
	 break;
      case '?':
      default:
	 if (!strcasecmp(pn, "poll"))
	    fprintf(stderr, "No options. It does everything for you.\n\n");
	 else if (!strcasecmp(pn, "report") || !strcasecmp(pn, "report.cgi"))
	    fprintf(stderr, report_usage);
	 exit(1);
      }

   if (!strcasecmp(pn, "report") && !recurse && !options.device) {
      fprintf(stderr, "%s", report_usage);
      exit(1);
   }
   read_conf_file(options.config_file, &defaults, &host_tree);

   if (!strcasecmp(pn, "poll"))
      poll_device_tree(&options, &defaults, host_tree);
   else if (!strcasecmp(pn, "report"))
      report_data(&defaults, &options, host_tree);
   else if (!strcasecmp(pn, "report.cgi")) {
      send_html_header(host_tree);
      get_cgi_params(&options);
      if (options.use_cgi)
	 report_data(&defaults, &options, host_tree);
      send_html_footer(&options);
   } else
      BAIL("Don't call me as %s. Use poll, report, or report.cgi", pn);

   if (options.show_parse) {
      dump_defaults(&defaults);
      dump_device_tree(host_tree);
   }
   return 0;
}
