/*
 * ocoloc.h - common header file
 *
 * $Id$
 *
 * Copyright (c) 2001, The Colocation Corporation. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.  Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution.  Neither the name of the The
 * Colocation Corporation nor the names of its contributors may be used to
 * endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * This software was originally written under grants from the Colocation
 * Corporation. To find the most recent versions of the software, please visit
 * http://www.coloco.com/public. To contribute patches or for other questions
 * please contact percentile@coloco.com
 *
 */

#ifndef __OCOLOC_H
#define __OCOLOC_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <time.h>
#include <assert.h>
#include <errno.h>
#include <syslog.h>
#include <gmp.h>
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>

#define DEBUG_SVS     	0x1
#define DEBUG_PARSE     0x2
#define DEBUG_POLL      0x4
#define DEBUG_RPT       0x8
#define DEBUG_CGI       0x10

#define NOW	time((time_t *)NULL)

typedef struct _defaults {
    char           *datapath;
    char           *community;
    char           *cgipath;
    float           percentage;
    unsigned long   syslog_fac;
}               Defaults;

typedef struct _stats {
    char           *hostname;
    char           *hostcomment;
    char           *ifname;
    char           *ifcomment;
    time_t          start;
    time_t          end;
    float           pct;
    unsigned int    samples;
    unsigned long   min_thru;
    unsigned long   max_thru;
    unsigned long   wgt_in;
    unsigned long   wgt_out;
    unsigned long   wgt_agg;
    mpz_t           avg_in;
    mpz_t           avg_out;
    mpz_t           ttl_in;
    mpz_t           ttl_out;
    char           *units;
}               Stats;

typedef struct _ifList {
    char           *name;
    char           *comment;

    FILE           *fp;
    float           pct;

    char           *OidName;
    oid             Oid[MAX_OID_LEN];
    int             OidLen;

    int             index;
    unsigned long   inOctets;
    time_t          inTime;
    unsigned long   outOctets;
    time_t          outTime;
    char            *upTime;

    Stats           stats;

    struct _entry  *data;
    struct _ifList *next;
}               IfList;

typedef struct _devList {
    char           *name;
    char           *comment;
    char           *community;

    struct _ifList *interface;
    struct _devList *next;
}               DevList;

typedef struct _entry {
    unsigned long   in;
    time_t          in_time;
    unsigned long   out;
    time_t          out_time;
    unsigned long   uptime;
    struct _entry  *next;
}               Entry;

enum {
bytes = 0, bits = -1, kilobytes = 1} unit_type;

typedef struct _options {
    char           *config_file;
    time_t          start;
    time_t          end;
    char           *device;
    char           *interface;
    char           *cmt;
    int             units;
    char           *unit_name;
    int             dont_fork;
    int             show_parse;
    int             use_cgi;
}               Options;

/*** DEFINES ***/

extern unsigned long gvBailSysLogPri;

#define	BAIL(fmt, va...) \
	do { \
        char errmsg[128]; \
        sprintf(errmsg, fmt, ## va); \
	fprintf(stderr, __FUNCTION__ "():%d %s\n", __LINE__, errmsg); \
	syslog(LOG_ERR | gvBailSysLogPri, "%s", errmsg); \
	exit(1); } while (0)

extern int      Debug;

/*** FUNCTIONS ***/

int             read_conf_file(char *, Defaults *, DevList **);
void            dump_defaults(Defaults *);
void            dump_device_tree(DevList *);

void            comment_to_interface(Options *, DevList *);

int             poll_device_tree(Options *, Defaults *, DevList *);
void            report_data(Defaults *, Options *, DevList *);
void            get_cgi_params(Options *);
void            send_html_header(DevList *);
void            send_html_footer(Options *);
void            write_html_stats(Stats *);

#endif				/* __OCOLOC_H */
